#!/usr/bin/env python2
# encoding: UTF-8
from __future__ import division, absolute_import, print_function, unicode_literals

import sys
import json
import codecs
import operator
import itertools
import pprint
import time
import datetime

import random

import cProfile

import SentimentAnalyzer

def parse_args():
    import argparse
    parser = argparse.ArgumentParser()

    parser.add_argument("json", type=argparse.FileType("r"))

    parser.add_argument("--limit", type=int, required=False)
    parser.add_argument("--profile", action="store_true")

    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("--cross-validate", metavar="N", type=int)
    group.add_argument("--validate", action="store_true")

    return parser.parse_args()

def split_list(lst, num_chunks):
    chunk_size = len(lst) // num_chunks

    # it's better to have slightly larger chunk 
    # than additional very small one
    if len(lst) % num_chunks:
        chunk_size += 1

    for i in xrange(0, len(lst), chunk_size):
        yield lst[i : i+chunk_size]

def select_chunk(chunks, index):
    return (
        chunks[index],
        reduce(operator.add, (chunk for i, chunk in enumerate(chunks) if i != index))
    )

def check_model(analyzer, control_data):
    return sum(int(x == y) for x, y in itertools.izip(
        analyzer.getClasses(x["text"] for x in control_data), 
        (x["polarity"] for x in control_data)
    )) / len(control_data)

def print_additional_analyzer_info(analyzer):
    if hasattr(analyzer._pipeline, "grid_scores_"):
        scores = sorted(analyzer._pipeline.grid_scores_, key=lambda x: x.mean_validation_score, reverse=False)
        pprint.pprint(scores)

    print()

def cross_validate(corpus, n):
    chunks = list(split_list(corpus, n))

    assert sum(len(chunk) for chunk in chunks) == len(corpus)
    assert len(chunks) == n

    stats = []

    for i in xrange(len(chunks)):
        chunk, rest = select_chunk(chunks, i)

        analyzer = SentimentAnalyzer.SentimentAnalyzer()
        analyzer.train(rest)

        rate = check_model(analyzer, chunk)        

        print("{rate:.4%} - {i}".format(i=i+1, rate=rate))
        print_additional_analyzer_info(analyzer)
        stats.append(rate)

    print("{:.4%} - average".format(sum(stats) / len(stats)))

def validate(corpus):
    analyzer = SentimentAnalyzer.SentimentAnalyzer()
    analyzer.train(corpus)
    
    rate = check_model(analyzer, corpus)

    print("{:.4%}".format(rate))
    print_additional_analyzer_info(analyzer)

def main():
    args = parse_args()
    #print(args)
    sys.stdout = codecs.getwriter(sys.stdout.encoding)(sys.stdout, "replace")

    corpus = json.load(args.json)
    
    rnd = random.Random()
    rnd.seed(42)
    rnd.shuffle(corpus)

    if args.limit >= 0:
        corpus = corpus[:args.limit]

    if args.profile:
        pr = cProfile.Profile()
        pr.enable()

    start_time = time.time()

    if args.cross_validate:
        cross_validate(corpus, args.cross_validate)
    elif args.validate:
        validate(corpus)

    if args.profile:
        pr.disable()
        pr.dump_stats("profile.prof")

    print("Time elapsed: ", datetime.timedelta(seconds=time.time() - start_time))

if __name__ == "__main__":
    main()
