# encoding: UTF-8
from __future__ import division, absolute_import, print_function, unicode_literals

import sklearn.base

import nltk
import nltk.stem.snowball

from . import tweet_processing
from . import util

class Extractor(sklearn.base.BaseEstimator):

    def __init__(self, 
            smile_weight=2.25, 
            ngram_weight=0.4, 
            hashtag_weight=1.0,
            capslock_weight=1.5,
            punctuation_weight=0.75,
            url_weight=1.0,
            ):
        super(Extractor, self).__init__()

        self.smile_weight = smile_weight
        self.ngram_weight = ngram_weight
        self.hashtag_weight = hashtag_weight
        self.capslock_weight = capslock_weight
        self.punctuation_weight = punctuation_weight
        self.url_weight = url_weight

        self._stemmer = util.CachedRussianStemmer()
    
    def _get_features(self, text):
        tweet_data = tweet_processing.preprocess_tweet(text)

        tokens = nltk.word_tokenize(tweet_data.text)
        word_tokens = [tok for tok in tokens if tweet_processing.check_word(tok)]

        for token in tokens:
            if len(token) <= 2:
                continue
            #if not tweet_processing.check_word(token):
            #    continue

            weight = 1.0
            token = token.lower() 

            if tweet_processing.check_russian_word(token):
                yield self._stemmer.stem(token), weight 
            else:
                yield token, weight

        for n in (2, 3, 4, 5):
            weight = self.ngram_weight

            for ngram in nltk.util.ngrams(word_tokens, n):
                if all(tweet_processing.check_russian_word(x) for x in ngram):
                    yield " ".join(self._stemmer.stem(tok) for tok in ngram).lower(), weight
                else:
                    yield " ".join(ngram).lower(), weight

        for hashtag in tweet_data.hashtags:
            yield hashtag.lower(), self.hashtag_weight

        for _ in tweet_data.happy_smilies:
            yield ":-)", self.smile_weight

        for _ in tweet_data.sad_smilies:
            yield ":-(", self.smile_weight

        for x in tweet_data.other_smilies:
            yield x, self.smile_weight

        for x in tweet_data.punctuation:
            yield x, self.punctuation_weight

        for url in tweet_data.urls:
            res = tweet_processing.parse_url(url)
            if res:
                yield res.netloc.lower(), self.url_weight

        yield "<NUMBER OF URLS>", len(tweet_data.urls)
        yield "<NUMBER OF MENTIONS>", len(tweet_data.mentions)
        yield "<NUMBER OF HASHTAGS>", len(tweet_data.hashtags)
        #yield "<RAW TWEET LENGTH>", len(tweet_data.raw_text)

        try:
            capslock_factor = sum(int(x.isupper()) for x in tweet_data.text) / sum(int(x.isalpha()) for x in tweet_data.text)
        except ZeroDivisionError:
            pass
        else:
            yield "<SPECIAL MAGIC CAPSLOCK FACTOR>", capslock_factor * self.capslock_weight

    def fit(self, texts, classes=None):
        return self
    partial_fit = fit

    def transform(self, texts):
        return (self._get_features(text) for text in texts)

