# coding=utf-8
from collections import Counter
import math

from nltk import ngrams
import nltk
from nltk.classify import SklearnClassifier
from sklearn import svm
from sklearn.naive_bayes import BernoulliNB

from happytokenizer import Tokenizer


__author__ = 'aleksandrov'

good_words = [u'хорош', u'молод', u'приятн', u'супер',
              u'крут', u'офиген', u'прекрасн', u'спасибо',
              u'охуительн', u'шикарн', u'класс', u'удовол',
              u'положительн', u'мил', u'здорово', u'довольн',
              u'восхитительн', u'благодар', u'отличн', u'добр',
              u'радост', u'счаст', u'лучши', u'лучша', u'лучшо',
              u'ура', u'еху', u'раду', u'божеств', u' любл', u'охренител',
              u'нереальн'
]
bad_words = [u'плох', u'ужас', u'пизд', u'бляд',
             u'наху', u'хуй', u'кал', u'груст',
             u'ненави', u'туп', u'дерьм', u'идиот', u'гад'
             u'гацк', u'имбецил', u'бесит', u'убожеств', u'ущербн',
             u'отстой', u'не раду', u'долбан', u'задолба', u'разочарован'
             u'печаль', u'достал', u'напряга', u'не любл'
]


class SentimentAnalyzer:
    def __init__(self):
        self.feature_function = self.get_features
        self.CLASS_POS = 'positive'
        self.CLASS_NEG = 'negative'
        self.CLASS_NEUT = 'neutral'
        self.tokenizer = Tokenizer()

    def weight(self, features):
        return dict([(feature, self.weight_delta_tfidf(feature, count)) for feature, count in features.items()])

    def get_features(self, item, tokenize=False):
        return self.features_grams_symbols(item, tokenize)

    def train_with_function(self, training_corpus, feature_function):
        self.feature_function = feature_function
        self.train(training_corpus)

    def weight_features(self, item, tokenize=False):
        return self.weight(self.feature_function(item, tokenize))

    def train(self, training_corpus):
        self.prepare(training_corpus)

        tweets = []
        for jsonObject in training_corpus:
            tweet = jsonObject['text']
            sentiment = jsonObject['polarity']
            words_filtered = [e.lower() for e in self.tokenizer.tokenize(tweet)]
            tweets.append((words_filtered, sentiment))

        training_set = nltk.classify.apply_features(self.weight_features, tweets)
        self.classifier = SklearnClassifier(svm.LinearSVC()).train(training_set)

    def getClasses(self, texts):
        classes = []
        for text in texts:
            splitted_text = self.tokenizer.tokenize(text)
            class_from_smiles = self.tokenizer.get_possible_sentiment_from_arr(splitted_text)
            if class_from_smiles != "none":
                classes.append(class_from_smiles)
                continue
            count_bad_words = self.count_text(splitted_text, 'bad')
            count_good_words = self.count_text(splitted_text, 'good')
            if count_bad_words > count_good_words:
                classes.append(self.CLASS_NEG)
                continue
            elif count_good_words > count_bad_words:
                classes.append(self.CLASS_POS)
                continue
            classes.append(self.classifier.classify(self.weight_features(splitted_text)))
        return classes

    def count_text(self, words, polarity):
        if polarity == 'bad':
            checked_words = bad_words
        else:
            checked_words = good_words
        text = ' '.join(words)
        count = 0
        for w in checked_words:
            count += text.count(w)
        return count


    def prepare(self, training_corpus):
        data = {}
        n_pos = 0
        n_neg = 0
        n_neut = 0

        for item in training_corpus:
            item_class = item['polarity']
            if item_class == self.CLASS_POS:
                n_pos += 1
                df_class = 'df_pos'
            elif item_class == self.CLASS_NEG:
                n_neg += 1
                df_class = 'df_neg'
            else:
                n_neut += 1
                df_class = 'df_neut'

            ngrams = self.feature_function(item['text'], tokenize=True)
            for ngram, count in ngrams.items():
                if ngram not in data:
                    data[ngram] = {
                        'df': 0,
                        'tf': [],
                        'df_pos': 0,
                        'df_neg': 0,
                        'df_neut': 0
                    }

                data[ngram]['df'] += 1
                data[ngram]['tf'].append(count)
                data[ngram][df_class] += 1
        self.index = data
        self.n_pos = n_pos
        self.n_neg = n_neg

    def features_ngrams(self, item, ngram_order, tokenize=False):
        return Counter(self.ngrams(item, ngram_order, tokenize))

    def features_ngrams_symbols(self, item, ngram_order):
        return Counter(self.ngrams_symbols(item, ngram_order))
        # return self.ngrams_symbols(item, ngram_order)

    def features_ngrams_1(self, item, tokenize=False):
        return self.features_ngrams(item, 1, tokenize)

    def features_ngrams_2(self, item, tokenize=False):
        return self.features_ngrams(item, 2, tokenize)

    def features_grams(self, item, tokenize=False):
        return self.features_ngrams_1(item, tokenize) + self.features_ngrams_2(item, tokenize)

    def features_grams_symbols(self, item, tokenize=False):
        if not tokenize:
            text = ' '.join(item)
        else:
            words = self.tokenizer.tokenize(item)
            text = ' '.join(words)
        return self.features_ngrams_symbols(text, 3) \
               + self.features_ngrams_symbols(text, 4) \
               + self.features_ngrams_symbols(text, 5)

    def ngrams(self, words, order, tokenize=False, sep=' '):
        if tokenize:
            words = self.tokenizer.tokenize(words)
        return [sep.join(words[n - order: n]) for n in range(order, len(words))]

    def ngrams_symbols(self, words, order, tokenize=False, sep=''):
        # if tokenize:
        #     words = self.tokenizer.tokenize(words)
        # return [sep.join(words[n - order: n]) for n in range(order, len(words))]
        grams = ngrams(words, order)
        return [sep.join(gram) for gram in grams]

    def weight_delta_idf(self, ngram, count):
        if (self.index.has_key(ngram)):
            a = self.index[ngram]['df_pos'] * self.n_neg
            b = self.index[ngram]['df_neg'] * self.n_pos
            if a <= 0:
                a = 0.5
            if b <= 0:
                b = 0.5
        else:
            a = 0.5
            b = 0.5
        return math.log(a / float(b)) if ngram in self.index else 0

    def weight_delta_tfidf(self, ngram, count):
        return count * self.weight_delta_idf(ngram, count)

    def avg(self, values):
        return sum(values) / len(values)
