#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import htmlentitydefs

emoticon_string = r"""
    (?:
      [<>]?
      [:;=8]                     # eyes
      [\-o\*\']?                 # optional nose
      [\*\)\]\(\[3dDpP/\:\}\{@\|\\] # mouth
      |
      [\)\]\(\[CdDpP/\:\}\{@\|\\] # mouth
      [\-o\*\']?                 # optional nose
      [:;=8]                     # eyes
      [<>]?
      |
      [\)]{1,}
      |
      [\(]{1,}
      |
      [\!]{2,}
      |
      [!]{1,}[?]{1,}[!?]{0,}
      |
      [?]{1,}[!]{1,}[!?]{0,}
      |
      [\^][\_]{0,}[\^]
    )"""

bad_emotions = r"""
    (?:
      [<>]?
      [:;=8]                     # eyes
      [\-o\*\']?                 # optional nose
      [\(\[C/\{@\|\\]             # mouth
      |
      [\)\]/\}@\|\\]             # mouth
      [\-o\*\']?                 # optional nose
      [:;=8]                     # eyes
      [<>]?
      |
      [\(]{1,}
      |
      [!]{1,}[?]{1,}[!?]{0,}
      |
      [?]{1,}[!]{1,}[!?]{0,}
    )"""

good_emotions = r"""
    (?:
      [<>]?
      [:;=8]                     # eyes
      [\-o\*\']?                 # optional nose
      [\*\)\]\3dDpP}@]             # mouth
      |
      [\(\[\{@]             # mouth
      [\-o\*\']?                 # optional nose
      [:;=8]                     # eyes
      [<>]?
      |
      [\!]{2,}
      |
      [\)]{1,}
      |
      [\^][\_]{0,}[\^]
    )"""

hyperlinks_re = r"""https?://[^\s<>"]+|www\.[^\s<>"]+"""
twitter_pic_re = r"""pic\.twitter\.com/[\w]+"""

regex_strings = (
    # Phone numbers:
    r"""
    (?:
      (?:            # (international)
        \+?[01]
        [\-\s.]*
      )?            
      (?:            # (area code)
        [\(]?
        \d{3}
        [\-\s.\)]*
      )?    
      \d{3}          # exchange
      [\-\s.]*   
      \d{4}          # base
    )"""
    ,
    # Emoticons:
    emoticon_string
    ,
    hyperlinks_re
    ,
    twitter_pic_re
    ,
    # HTML tags:
     r"""<[^>]+>"""
    ,
    # Twitter username:
    r"""(?:@[\w_]+)"""
    ,
    # Twitter hashtags:
    r"""(?:\#+[\w_]+[\w\'_\-]*[\w_]+)"""
    ,
    # Remaining word types:
    r"""
    (?:[a-z][a-z'\-_]+[a-z])       # Words with apostrophes or dashes.
    |
    (?:[+\-]?\d+[,/.:-]\d+[+\-]?)  # Numbers, including fractions, decimals.
    |
    (?:[\w_]+)                     # Words without apostrophes or dashes.
    |
    (?:\.(?:\s*\.){1,})            # Ellipsis dots. 
    |
    (?:\S)                         # Everything else that isn't whitespace.
    """
    )
    
word_re = re.compile(r"""(%s)""" % "|".join(regex_strings), re.VERBOSE | re.I | re.UNICODE)

emoticon_re = re.compile(regex_strings[1], re.VERBOSE | re.I | re.UNICODE)
good_emotions_re = re.compile(good_emotions, re.VERBOSE | re.I | re.UNICODE)
bad_emotions_re = re.compile(bad_emotions, re.VERBOSE | re.I | re.UNICODE)

html_entity_digit_re = re.compile(r"&#\d+;")
html_entity_alpha_re = re.compile(r"&\w+;")
amp = "&amp;"

class Tokenizer:
    def __init__(self, preserve_case=False):
        self.preserve_case = preserve_case

    def tokenize(self, s):
        try:
            s = unicode(s)
        except UnicodeDecodeError:
            s = str(s).encode('string_escape')
            s = unicode(s)
        # Fix HTML character entitites:
        s = self.__html2unicode(s)
        # Tokenize:
        words = word_re.findall(s)
        words2 = []
        for i in range(len(words)):
            t = words[i]
            if len(t) < 3 :
                if re.match(emoticon_re, t):
                    words2.append(t)
                    continue
                continue
            if not re.match(hyperlinks_re, t) and not re.match(twitter_pic_re, t):
                words2.append(t)
        words = words2
        # Possible alter the case, but avoid changing emoticons like :D into :d:
        if not self.preserve_case:            
            words = map((lambda x : x if emoticon_re.search(x) else x.lower()), words)
        return words

    def get_possible_sentiment_from_text(self, s):
        try:
            s = unicode(s)
        except UnicodeDecodeError:
            s = str(s).encode('string_escape')
            s = unicode(s)
        good_emotions = good_emotions_re.findall(s)
        bad_emotions = bad_emotions_re.findall(s)
        good_count = len(good_emotions)
        bad_count = len(bad_emotions)
        if good_count > 0 :
            if bad_count < good_count:
                return "positive"
            elif bad_count == good_count:
                return "none"
            else:
                return "negative"
        elif bad_count > 0:
            return "negative"
        else:
            return "none"

    def get_possible_sentiment_from_arr(self, s):
        text = ''
        for word in s:
            text = text + ' ' + word
        return self.get_possible_sentiment_from_text(text)

    def __html2unicode(self, s):
        ents = set(html_entity_digit_re.findall(s))
        if len(ents) > 0:
            for ent in ents:
                entnum = ent[2:-1]
                try:
                    entnum = int(entnum)
                    s = s.replace(ent, unichr(entnum))	
                except:
                    pass
        # Now the alpha versions:
        ents = set(html_entity_alpha_re.findall(s))
        ents = filter((lambda x : x != amp), ents)
        for ent in ents:
            entname = ent[1:-1]
            try:            
                s = s.replace(ent, unichr(htmlentitydefs.name2codepoint[entname]))
            except:
                pass                    
            s = s.replace(amp, " and ")
        return s
